package com.chinaitop.depot.monitor.service.quartzJob;

import com.google.gson.Gson;
import com.chinaitop.depot.monitor.model.MonitorJobEnty;
import com.chinaitop.depot.monitor.service.quartzJob.Constant.MonitorStatus;

import org.quartz.*;

/**
 * 定时任务工具类
 * 
 * @author hf
 * @date 2018年6月29日 下午13:15:59
 */
public class MonitorUtils {
    private final static String JOB_NAME = "TASK_";
    
    /**
     * 获取触发器key
     */
    public static TriggerKey getTriggerKey(String jobId) {
    	return TriggerKey.triggerKey(JOB_NAME + jobId);
    }
    
    /**
     * 获取jobKey
     */
    public static JobKey getJobKey(String jobId) {
    	return JobKey.jobKey(JOB_NAME + jobId);
    }
    
    /**
     * 获取表达式触发器
     */
    public static CronTrigger getCronTrigger(Scheduler scheduler, String jobId) {
    	try {
    		return (CronTrigger) scheduler.getTrigger(getTriggerKey(jobId));
    	} catch (SchedulerException e) {
    		throw new RuntimeException("获取定时任务CronTrigger出现异常", e);
    	}
    }
    
    /**
     * 创建定时任务
     */
    public static void createMonitorJob(Scheduler scheduler, MonitorJobEnty monitorJob) {
    	try {
    		//构建job信息
    		JobDetail jobDetail = JobBuilder.newJob(MonitorJobExecuter.class).withIdentity(getJobKey(monitorJob.getId())).build();
    		
    		//表达式调度构建器
    		CronScheduleBuilder scheduleBuilder = CronScheduleBuilder.cronSchedule(monitorJob.getCronExpression())
    				.withMisfireHandlingInstructionDoNothing();
    		
    		//按新的cronExpression表达式构建一个新的trigger
    		CronTrigger trigger = TriggerBuilder.newTrigger().withIdentity(getTriggerKey(monitorJob.getId())).withSchedule(scheduleBuilder).build();
    		
    		//放入参数，运行时的方法可以获取
    		jobDetail.getJobDataMap().put(MonitorJobEnty.JOB_PARAM_KEY, new Gson().toJson(monitorJob));
    		
    		scheduler.scheduleJob(jobDetail, trigger);
    		
    		//暂停任务
    		if(monitorJob.getStatus() == MonitorStatus.PAUSE.getValue()){
    			pauseJob(scheduler, monitorJob.getId());
    		}
    	} catch (SchedulerException e) {
    		throw new RuntimeException("创建定时任务失败", e);
    	}
    }
    
    /**
     * 更新定时任务
     */
    public static void updateMonitorJob(Scheduler scheduler, MonitorJobEnty monitorJob) {
    	try {
    		TriggerKey triggerKey = getTriggerKey(monitorJob.getId());
    		
    		//表达式调度构建器
    		CronScheduleBuilder scheduleBuilder = CronScheduleBuilder.cronSchedule(monitorJob.getCronExpression())
    				.withMisfireHandlingInstructionDoNothing();
    		
    		CronTrigger trigger = getCronTrigger(scheduler, monitorJob.getId());
    		
    		//按新的cronExpression表达式重新构建trigger
    		trigger = trigger.getTriggerBuilder().withIdentity(triggerKey).withSchedule(scheduleBuilder).build();
    		
    		//参数
    		trigger.getJobDataMap().put(MonitorJobEnty.JOB_PARAM_KEY, new Gson().toJson(monitorJob));
    		
    		scheduler.rescheduleJob(triggerKey, trigger);
    		
    		//暂停任务
    		if(monitorJob.getStatus() == MonitorStatus.PAUSE.getValue()){
    			pauseJob(scheduler, monitorJob.getId());
    		}
    		
    	} catch (SchedulerException e) {
    		throw new RuntimeException("更新定时任务失败", e);
    	}
    }
    
    /**
     * 立即执行任务
     */
    public static void run(Scheduler scheduler, MonitorJobEnty monitorJob) {
    	try {
    		//参数
    		JobDataMap dataMap = new JobDataMap();
    		dataMap.put(MonitorJobEnty.JOB_PARAM_KEY, new Gson().toJson(monitorJob));
    		
    		scheduler.triggerJob(getJobKey(monitorJob.getId()), dataMap);
    	} catch (SchedulerException e) {
    		throw new RuntimeException("立即执行定时任务失败", e);
    	}
    }
    
    /**
     * 暂停任务
     */
    public static void pauseJob(Scheduler scheduler, String jobId) {
    	try {
    		scheduler.pauseJob(getJobKey(jobId));
    	} catch (SchedulerException e) {
    		throw new RuntimeException("暂停定时任务失败", e);
    	}
    }

    /**
     * 恢复任务
     */
    public static void resumeJob(Scheduler scheduler, String jobId) {
        try {
            scheduler.resumeJob(getJobKey(jobId));
        } catch (SchedulerException e) {
            throw new RuntimeException("暂停定时任务失败", e);
        }
    }

    /**
     * 删除定时任务
     */
    public static void deleteScheduleJob(Scheduler scheduler, String jobId) {
        try {
            scheduler.deleteJob(getJobKey(jobId));
        } catch (SchedulerException e) {
            throw new RuntimeException("删除定时任务失败", e);
        }
    }
}
